/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: MailBCJBIListener.java 154 27 sept. 06 ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.mail.listeners;

import java.util.logging.Level;

import javax.jbi.messaging.MessagingException;
import javax.mail.Session;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.ow2.petals.bc.mail.Constants;
import org.ow2.petals.bc.mail.MailBCImpl;
import org.ow2.petals.bc.mail.service.SendMailService;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;

/**
 * Mail BC listener for incoming JBI messages
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class MailBCJBIListener extends AbstractJBIListener {

    protected MailSessionManager mailSessionManager;

    protected MimeMessageManager mimeMessageManager;

    protected SessionDescriptorBuilder sessionDescriptorBuilder;

    protected SendMailService sendMailService;
    
    @Override
    protected void init() {
        this.mimeMessageManager = ((MailBCImpl) getComponent()).getMimeMessageManager();
        this.mailSessionManager = ((MailBCImpl) getComponent()).getMailSessionManager();
        this.sessionDescriptorBuilder = ((MailBCImpl) getComponent()).getSessionDescriptorBuilder();
        this.sendMailService= new SendMailService(mailSessionManager,mimeMessageManager,sessionDescriptorBuilder, getLogger());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.objectweb.petals.component.common.bc.JBIListener#onJBIMessage(java
     * .lang.String,
     * org.objectweb.petals.component.common.util.MessageExchangeWrapper,
     * org.objectweb.petals.tools.jbicommon.descriptor.Extensions)
     */
    public void onJBIMessage(Exchange exchange) {

        // Fault ignored
        if (exchange.getFault() == null) {
            if (exchange.getOperation() !=  null && Constants.SEND_MAIL_OPERATION.equalsIgnoreCase(exchange.getOperation()
                    .getLocalPart())) {
                sendMailService.sendMailService(exchange,getExtensions());
            } else {
                standardSendService(exchange);
            }
        }
    }

    /**
     * Standard service : send the jbi message content as an email<br>
     * the email address and subject are defined in the srv unit or in the
     * "WSA-TO", "WSA-ACTION" jbi message properties
     * 
     * @param exchange
     */
    protected void standardSendService(Exchange exchange) {
        try {
            getLogger().log(Level.INFO, "JBI Exchange received - send it as a mail");

            ConfigurationExtensions extensions = getExtensions();

            // Build a mail session descriptor from the given address (URI)
            SessionDescriptor sessionDescriptor = sessionDescriptorBuilder.build(exchange,
                    extensions);

            // Build the destination address
            InternetAddress internetAddress = new InternetAddress(sessionDescriptor.getToAddress());

            // Build the mail session
            Session session = mailSessionManager
                    .createSessionPropertiesFromDescriptor(sessionDescriptor);

            // Create a mail message with the incoming jbi message
            MimeMessage mimeMessage = mimeMessageManager.mapNormalizedMessageToMimeMessage(session,
                    internetAddress, sessionDescriptor.getSubject(), exchange.getInMessage());

            // Send this mail
            mailSessionManager.sendMail(mimeMessage, sessionDescriptor, session);

            getLogger().log(Level.INFO, "email sent to " + sessionDescriptor.getToAddress());

        } catch (Throwable e) {
            try {
                if (!exchange.isInOnlyPattern()) {
                    exchange.setFault(new Exception(e));
                } else {
                    getLogger().log(Level.SEVERE, e.getMessage(), e);
                }
            } catch (MessagingException e1) {
                getLogger().log(Level.SEVERE, "Can't return fault to consumer", e);
            }
        }
    }

  
}
